import math

from geometry_msgs.msg import Point, Pose, Quaternion
from trajectory_msgs.msg import JointTrajectoryPoint


leg_offsets = {    
    'e1': Point(x = -40.0, y =  80.0, z = 25.0),
    'e2': Point(x = -60.0, y =   0.0, z = 25.0),
    'e3': Point(x = -40.0, y = -80.0, z = 25.0),
    'e4': Point(x =  40.0, y = -80.0, z = 25.0),
    'e5': Point(x =  60.0, y =   0.0, z = 25.0),
    'e6': Point(x =  40.0, y =  80.0, z = 25.0)
}

leg_sides = {
    'e1': 'l',
    'e2': 'l',
    'e3': 'l',
    'e4': 'r',
    'e5': 'r',
    'e6': 'r',    
}


def ik(body_pose):
    # todo apply rotation and translation of body

    joint_values = {}
    for l in body_pose['legs']:
        joint_values[l] = leg_ik(
            leg_sides[l], 
            Point(
                x = body_pose['legs'][l].x - leg_offsets[l].x - body_pose['body'].position.x,
                y = body_pose['legs'][l].y - leg_offsets[l].y - body_pose['body'].position.y,
                z = body_pose['legs'][l].z - leg_offsets[l].z - body_pose['body'].position.z,
            )            
        )
    
    return joint_values

def leg_ik(side, point):

    x = point.x
    y = point.y
    z = point.z

    c = 20
    f = 80
    t = 130
    
    ff = f*f
    tt = t*t

    if side == 'l':
        x = -x

    coxa_angle = math.atan2(y,x)
    
    r = math.sqrt(x*x + y * y) - c

    rr = r*r
    zz = z*z
    dd = rr + zz
    d  = math.sqrt(dd)
    
    femur_angle = math.acos((ff + dd - tt) / (2 * f * d)) + math.acos(-z/d) - math.pi/2
    tibia_angle = math.acos((tt + ff - dd) / (2 * t * f))                   - math.pi/2

    if(side == 'l'):
        coxa_angle *= -1
        femur_angle *= -1
    else:
        tibia_angle *= -1


    return([coxa_angle, femur_angle, tibia_angle])

if __name__ == "__main__":
    print(leg_ik('l', Point(x = -100.0, y = 0.0, z = -130.0)))
    print(leg_ik('r', Point(x =  100.0, y = 0.0, z = -130.0)))
    print(leg_ik('l', Point(x = -120.0, y = 0.0, z = -130.0)))
    print(leg_ik('r', Point(x =  120.0, y = 0.0, z = -130.0)))

    for o in leg_offsets:
        print(leg_offsets[o])


    print(ik(standing_pose))
    print(ik(shutdown_pose))

    
    poses = interpolate_pose(shutdown_pose, standing_pose , 10)
    for p in poses:
        print(p)
        print(ik(p))
        print()
